@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2018 David J. Barnes
+ *  Copyright (C) 1994-2019 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -56,7 +56,8 @@ static Boolean apply_variations(const Game *game_details, const Board *board,
 static Boolean rewrite_variations(const Board *board, Variation *variation);
 static Boolean rewrite_moves(Game *game, Board *board, Move *move_details);
 static void build_FEN_components(const Board *board, char *epd, char *fen_suffix);
-static Boolean drop_plys(Game *game, Move *moves, unsigned plys_to_drop);
+static unsigned plies_in_move_sequence(Move *moves);
+static Boolean drop_plies_from_start(Game *game, Move *moves, int plies_to_drop);
 #if 0
 static void append_evaluation(Move *move_details, const Board *board);
 static void append_FEN_comment(Move *move_details, const Board *board);
@@ -158,10 +159,15 @@ char *
 copy_string(const char *str)
 {
     char *result;
-    size_t len = strlen(str);
+    if(str != NULL) {
+        size_t len = strlen(str);
 
-    result = (char *) malloc_or_die(len + 1);
-    strcpy(result, str);
+        result = (char *) malloc_or_die(len + 1);
+        strcpy(result, str);
+    }
+    else {
+        result = NULL;
+    }
     return result;
 }
 
@@ -970,7 +976,7 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
 
                     if (GlobalState.check_for_repetition) {
                         Boolean repetition =
-                            update_position_counts(game_details->position_counts, board->weak_hash_value);
+                            update_position_counts(game_details->position_counts, board);
                         if (repetition && GlobalState.add_position_match_comments) {
                             CommentList *comment = create_match_comment(board);
                             append_comments_to_move(next_move, comment);
@@ -1070,23 +1076,23 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
                         const char *result_tag = game_details->tags[RESULT_TAG];
                         if (result_tag != NULL && move_result != NULL &&
                                 strcmp(result_tag, move_result) != 0) {
-                            // Mismatch.
-                            // Whether to report it.
+                            /* Mismatch. */
+                            /* Whether to report it. */
                             Boolean report = TRUE;
                             if (GlobalState.fix_result_tags) {
                                 if(strcmp(move_result, "*") == 0 || 
                                         strcmp(result_tag, "*") == 0) {
-                                    // Prefer the move result.
+                                    /* Prefer the move result. */
                                     free((void *) result_tag);
                                     game_details->tags[RESULT_TAG] = copy_string(move_result);
                                     report = FALSE;
                                 }
                                 else {
-                                    // Irreconcilable conflict.
+                                    /* Irreconcilable conflict. */
                                 }
                             }
                             else if(strcmp(move_result, "*") == 0) {
-                                // Don't report this form of inconsistency.
+                                /* Don't report this form of inconsistency. */
                                 report = FALSE;
                             }
                             if(report) {
@@ -1247,8 +1253,21 @@ play_moves(Game *game_details, Board *board, Move *moves, unsigned max_depth,
         /* Decide whether to keep it anyway. */
         if (GlobalState.keep_broken_games) {
         }
+#if 0
         else if (GlobalState.positional_variations) {
+            /* NB: Not rejecting the match when a game is not ok led to
+             * a memory bug in v18-10. This is now fixed so this code
+             * could be retained. However, ...
+             * I think I felt it appropriate to retain broken games
+             * when positional variations are being sought so that games
+             * to be truncated before the end would still be matched, but it
+             * has the consequences errors are reported twice and broken
+             * games may be counted as matches.
+             * For the time being, I am closing this route but it might
+             * be appropriate to re-open it at some point.
+             */
         }
+#endif
         else {
             /* Only return a match if it genuinely matched a variation
              * in which we were interested.
@@ -1390,7 +1409,7 @@ apply_move_list(Game *game_details, unsigned *plycount, unsigned max_depth)
     game_details->cumulative_hash_value = 0;
 
     if (GlobalState.check_for_repetition && game_details->position_counts == NULL) {
-        game_details->position_counts = new_position_count_list(board->weak_hash_value);
+        game_details->position_counts = new_position_count_list(board);
     }
 
     /* Play through the moves and see if we have a match.
@@ -1429,7 +1448,9 @@ apply_eco_move_list(Game *game_details, unsigned *number_of_half_moves)
     /* Start off the cumulative hash value. */
     game_details->cumulative_hash_value = 0;
     play_eco_moves(game_details, board, moves);
-    //game_details->moves_checked = TRUE;
+#if 0
+    game_details->moves_checked = TRUE;
+#endif
     /* Record how long the game was. */
     *number_of_half_moves = half_moves_played(board);
     free_board(board);
@@ -1673,7 +1694,10 @@ rewrite_move(Game *game, Colour colour, Move *move_details, Board *board)
 { /* Assume success. */
     Boolean Ok = TRUE;
     
-    if (GlobalState.output_format == SOURCE || 
+    if(move_details->class == UNKNOWN_MOVE) {
+        Ok = FALSE;
+    }
+    else if (GlobalState.output_format == SOURCE || 
             rewrite_SAN_string(colour, move_details, board)) {
         Piece piece_to_move = move_details->piece_to_move;
         MoveClass class = move_details->class;
@@ -1748,7 +1772,7 @@ rewrite_moves(Game *game, Board *board, Move *moves)
 {
     Boolean game_ok = TRUE;
     Move *move_details = moves;
-    unsigned plys_to_drop = GlobalState.drop_ply_number;
+    int plies_to_drop = GlobalState.drop_ply_number;
 
     while (game_ok && (move_details != NULL)) {
         if (*(move_details->move) != '\0') {
@@ -1784,9 +1808,9 @@ rewrite_moves(Game *game, Board *board, Move *moves)
                     if(find_matching_comment(GlobalState.drop_comment_pattern,
                                               move_details->comment_list) != NULL) {
                         /* We have a match. */
-                        plys_to_drop = (board->move_number - 1) * 2;
+                        plies_to_drop = (board->move_number - 1) * 2;
                         if(board->to_move == BLACK) {
-                            plys_to_drop++;
+                            plies_to_drop++;
                         }
                     }
                 }
@@ -1882,8 +1906,17 @@ rewrite_moves(Game *game, Board *board, Move *moves)
             }
         }
     }
-    else if(plys_to_drop > 0 && game != NULL) {
-        game_ok = drop_plys(game, moves, plys_to_drop);
+    else if(plies_to_drop != 0 && game != NULL) {
+        if(plies_to_drop < 0) {
+            unsigned plies = plies_in_move_sequence(moves);
+            plies_to_drop = plies + plies_to_drop;
+        }
+        if(plies_to_drop >= 0) {
+            game_ok = drop_plies_from_start(game, moves, plies_to_drop);
+        }
+        else {
+            game_ok = FALSE;
+        }
     }
     return game_ok;
 }
@@ -1932,49 +1965,66 @@ rewrite_game(Game *current_game)
     return board;
 }
 
-/* Drop the given number of play from this game.
+/* Return the number of plies in the given move
+ * sequence.
+ */
+static unsigned plies_in_move_sequence(Move *moves)
+{
+    int num_plies = 0;
+    while(moves != NULL) {
+        moves = moves->next;
+        num_plies++;
+    }
+    return num_plies;
+}
+
+/* Drop the given number of play from the start of this game.
+ * If plies_to_drop is negative, drop all but that number
+ * from the beginning.
  * Return FALSE if there are insufficient.
- * If there are sufficient, replace the game's FEN tag
- * with the revised starting state.
+ * If there are sufficient replace the game's FEN tag with
+ * the revised starting state.
  */
 static Boolean
-drop_plys(Game *game, Move *moves, unsigned plys_to_drop)
+drop_plies_from_start(Game *game, Move *moves, int plies_to_drop)
 {
     char *fen = game->tags[FEN_TAG];
     Board *board = new_game_board(fen);
     Boolean game_ok = TRUE;
-    unsigned plys = 0;
-    Move *move_details = moves;
+    int plies = 0;
+    Move *new_head = moves;
 
-    if(plys_to_drop > 0 && game->prefix_comment != NULL) {
+    if(plies_to_drop > 0 && game->prefix_comment != NULL) {
         /* Don't free the prefix_comment because that is
          * handled elsewhere.
          */
         game->prefix_comment = NULL;
     }
-    while(plys < plys_to_drop && move_details != NULL && game_ok) {
+    while(plies < plies_to_drop && new_head != NULL && game_ok) {
         /* Rewriting is not strictly necessary, because it has already been done,
          * but it provides all the required functionality.
          */
-        if (rewrite_move(game, board->to_move, move_details, board)) {
+        if (rewrite_move(game, board->to_move, new_head, board)) {
             board->to_move = OPPOSITE_COLOUR(board->to_move);
             if (board->to_move == WHITE) {
                 board->move_number++;
             }
-            plys++;
-            move_details = move_details->next;
+            plies++;
+            new_head = new_head->next;
         }
         else {
             game_ok = FALSE;
         }
     }
-    if(plys == plys_to_drop && move_details != NULL && game_ok) {
+    if(plies == plies_to_drop && game_ok) {
         if(fen != NULL) {
             (void) free((void *) fen);
         }
-        // Reset the move number.
+#if 0
+        /* Reset the move number. */
         board->move_number = 1;
-        game->moves = move_details;        
+#endif
+        game->moves = new_head;        
         game->tags[FEN_TAG] = get_FEN_string(board);
         game->tags[SETUP_TAG] = copy_string("1");
         if(game->tags[PLY_COUNT_TAG] != NULL || GlobalState.output_plycount) {
@@ -1985,17 +2035,21 @@ drop_plys(Game *game, Move *moves, unsigned plys_to_drop)
             add_total_plycount(game, GlobalState.keep_variations);
         }
 
-        /* Free the dropped moves. */
-        Move *move_to_drop = moves;
-        while(move_to_drop->next != move_details) {
-            move_to_drop = move_to_drop->next;
+        /* Make sure we aren't dropping 0 moves. */
+        if(new_head != moves) {
+            /* Free the dropped moves. */
+            Move *move_to_drop = moves;
+            while(move_to_drop->next != new_head) {
+                move_to_drop = move_to_drop->next;
+            }
+            move_to_drop->next = NULL;
+            free_move_list(moves);
         }
-        move_to_drop->next = NULL;
-        free_move_list(moves);
     }
     else {
         game_ok = FALSE;
     }
+    (void) free((void *) board);
     return game_ok;
 }
 
