@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2018 David J. Barnes
+ *  Copyright (C) 1994-2019 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
 #include "lists.h"
 #include "mymalloc.h"
 
-#define CURRENT_VERSION "v18-10"
+#define CURRENT_VERSION "v19-04"
 #define URL "https://www.cs.kent.ac.uk/people/staff/djb/pgn-extract/"
 
 /* The prefix of the arguments allowed in an argsfile.
@@ -166,7 +166,7 @@ usage_and_exit(void)
         "-vvariations -- the file variations contains the textual lines of interest.",
         "-V -- don't include variations in the output. Ordinarily these are retained.",
         "-wwidth -- set width as an approximate line width for output.",
-        "-W[cm|epd|halg|lalg|elalg|xlalg|san] -- specify the output format to use.",
+        "-W[cm|epd|halg|lalg|elalg|xlalg|xolalg|san] -- specify the output format to use.",
         "      Default is SAN.",
         "      -W means use the input format.",
         "      -Wcm is (a possibly obsolete) ChessMaster format.",
@@ -176,6 +176,7 @@ usage_and_exit(void)
         "      -Wlalg is long algebraic.",
         "      -Welalg is enhanced long algebraic.",
         "      -Wxlalg is enhanced long algebraic with x for captures and - for non capture moves.",
+        "      -Wxolalg is -Wxlalg but with O-O and O-O-O for castling.",
         "      -Wuci is output compatible with the UCI protocol.",
         "-xvariations -- the file variations contains the lines resulting in",
         "                positions of interest.",
@@ -238,6 +239,7 @@ usage_and_exit(void)
         "--totalplycount - include a tag with the total number of plies in a game.",
         "--underpromotion - match only games that contain an underpromotion.",
         "--version - print the current version number and exit.",
+        "--xroster - don't output tags not included with the -R option (see -R).",
 
         /* Must be NULL terminated. */
         (char *) NULL,
@@ -249,7 +251,7 @@ usage_and_exit(void)
             "pgn-extract %s (%s): a Portable Game Notation (PGN) manipulator.\n",
             CURRENT_VERSION, __DATE__);
     fprintf(GlobalState.logfile,
-            "Copyright (C) 1994-2018 David J. Barnes (d.j.barnes@kent.ac.uk)\n");
+            "Copyright (C) 1994-2019 David J. Barnes (d.j.barnes@kent.ac.uk)\n");
     fprintf(GlobalState.logfile, "%s\n\n", URL);
     fprintf(GlobalState.logfile, "Usage: pgn-extract [arguments] [file.pgn ...]\n");
 
@@ -840,13 +842,14 @@ process_argument(char arg_letter, const char *associated_value)
         }
             break;
         case SEVEN_TAG_ROSTER_ARGUMENT:
-            if (GlobalState.tag_output_format == ALL_TAGS ||
-                    GlobalState.tag_output_format == SEVEN_TAG_ROSTER) {
+            if ((GlobalState.tag_output_format == ALL_TAGS ||
+                 GlobalState.tag_output_format == SEVEN_TAG_ROSTER) &&
+                     !GlobalState.only_output_wanted_tags) {
                 GlobalState.tag_output_format = SEVEN_TAG_ROSTER;
             }
             else {
                 fprintf(GlobalState.logfile,
-                        "-%c clashes with another argument.\n",
+                        "-%c clashes with another roster-related argument.\n",
                         SEVEN_TAG_ROSTER_ARGUMENT);
                 exit(1);
             }
@@ -1039,14 +1042,14 @@ process_long_form_argument(const char *argument, const char *associated_value)
     }
     else if (stringcompare(argument, "dropply") == 0) {
         /* Extract the number. */
-        unsigned number = 0;
+        int number = 0;
 
-        if (sscanf(associated_value, "%u", &number) == 1) {
+        if (sscanf(associated_value, "%d", &number) == 1) {
             GlobalState.drop_ply_number = number;
         }
         else {
             fprintf(GlobalState.logfile,
-                    "--%s requires a positive number following it.\n", argument);
+                    "--%s requires a number following it.\n", argument);
             exit(1);
         }
         return 2;
@@ -1212,7 +1215,7 @@ process_long_form_argument(const char *argument, const char *associated_value)
         }
         else {
             fprintf(GlobalState.logfile,
-                    "--notags clashes with another argument.\n");
+                    "--notags clashes with another roster-related argument.\n");
             exit(1);
         }
         return 1;
@@ -1387,6 +1390,16 @@ process_long_form_argument(const char *argument, const char *associated_value)
         exit(0);
         return 1;
     }
+    else if (stringcompare(argument, "xroster") == 0) {
+        if(GlobalState.tag_output_format == SEVEN_TAG_ROSTER) {
+            fprintf(GlobalState.logfile,
+                    "--%s clashes with -%c.\n",
+                    argument, SEVEN_TAG_ROSTER_ARGUMENT);
+            exit(1);
+        }
+        GlobalState.only_output_wanted_tags = TRUE;
+        return 1;
+    }
     else {
         fprintf(GlobalState.logfile,
                 "Unrecognised long-form argument: --%s\n",
