@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2018 David J. Barnes
+ *  Copyright (C) 1994-2019 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,7 +59,6 @@ typedef struct ending_details {
      * both colours.
      */
     Boolean both_colours;
-    const char *expression;
     /* The number of each set of pieces. */
     int num_pieces[2][NUM_PIECE_VALUES];
     Occurs occurs[2][NUM_PIECE_VALUES];
@@ -119,14 +118,13 @@ is_English_piece(char c)
  * in the data.
  */
 static Ending_details *
-new_ending_details(const char *expression, Boolean both_colours)
+new_ending_details(Boolean both_colours)
 {
     Ending_details *details = (Ending_details *) malloc_or_die(sizeof (Ending_details));
     int c;
     Piece piece;
 
     details->both_colours = both_colours;
-    details->expression = copy_string(expression);
     for (piece = PAWN; piece <= KING; piece++) {
         for (c = 0; c < 2; c++) {
             details->num_pieces[c][piece] = 0;
@@ -323,6 +321,9 @@ extract_piece_information(const char *line, Ending_details *details, Colour colo
     }
 }
 
+/* Extract the piece specification from line and fill out
+ * details with the pattern information.
+ */
 static Boolean
 decompose_line(const char *line, Ending_details *details)
 {
@@ -699,7 +700,7 @@ process_ending_line(const char *line, Boolean both_colours)
     Boolean Ok = TRUE;
 
     if (non_blank_line(line)) {
-        Ending_details *details = new_ending_details(line, both_colours);
+        Ending_details *details = new_ending_details(both_colours);
 
         if (decompose_line(line, details)) {
             /* Add it on to the list. */
