@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2018 David J. Barnes
+ *  Copyright (C) 1994-2019 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -955,13 +955,16 @@ static Boolean consistent_FEN_tags(Game *current_game)
         if(board != NULL) {
             /* There must be a SETUP_TAG to go with it. */
             if(current_game->tags[SETUP_TAG] == NULL) {
+                // This is such a common problem that it makes
+                // more sense just to silently correct it.
+#if 0
                 report_details(GlobalState.logfile);
                 fprintf(GlobalState.logfile,
                         "Missing %s Tag to accompany %s Tag.\n",
                         tag_header_string(SETUP_TAG),
                         tag_header_string(FEN_TAG));
                 print_error_context(GlobalState.logfile);
-
+#endif
                 /* Fix the inconsistency. */
                 current_game->tags[SETUP_TAG] = copy_string("1");
             }
@@ -1030,7 +1033,7 @@ deal_with_game(Move *move_list)
      * previous_occurrence function.
      *
      * If there are any tag criteria, it will be easy to quickly
-     * eliminate most games without going through the length
+     * eliminate most games without going through the lengthy
      * process of game matching.
      *
      * If ECO adding is done, the order of checking may cause
