@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2018 David J. Barnes
+ *  Copyright (C) 1994-2019 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -104,7 +104,7 @@ StateInfo GlobalState = {
     -1,                 /* output_ply_limit (--plylimit) */
     0,                  /* stability_threshold (--stable) */
     0,                  /* maximum_matches */
-    0,                  /* drop_ply_number */
+    0,                  /* drop_ply_number (--dropply) */
     FALSE,              /* output_FEN_string */
     FALSE,              /* add_FEN_comments (--fencomments) */
     FALSE,              /* add_hashcode_comments (--hashcomments) */
@@ -120,6 +120,7 @@ StateInfo GlobalState = {
     FALSE,              /* allow_nested_comments (--nestedcomments) */
     FALSE,              /* add_match_tag (--addmatchtag) */
     FALSE,              /* add_matchlabel_tag (--addlabeltag) */
+    FALSE,              /* only_output_wanted_tags (--xroster) */
     0,                  /* split_depth_limit */
     NORMALFILE,         /* current_file_type */
     SETUP_TAG_OK,       /* setup_status */
