@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2018 David J. Barnes
+ *  Copyright (C) 1994-2019 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -395,6 +395,10 @@ make_move(MoveClass class, Col from_col, Rank from_rank, Col to_col, Rank to_ran
     board->board[to_r][to_c] = MAKE_COLOURED_PIECE(colour, piece);
     /* Insert the moved piece into the hash value. */
     board->weak_hash_value ^= hash_lookup(to_col, to_rank, piece, colour);
+    if(!board->EnPassant) {
+        board->ep_rank = '\0';
+        board->ep_col = '\0';
+    }
 
     if (castling_rook_col != '\0') {
         /* The rook involved in the castling move must now be moved. */
@@ -2006,6 +2010,10 @@ determine_move_details(Colour colour, Move *move_details, Board *board)
             }
         }
     }
+    if(!Ok) {
+        /* Treat invalid moves as being of an unknown move class. */
+        move_details->class = UNKNOWN_MOVE;
+    }
     return Ok;
 }
 
