@@ -1,6 +1,6 @@
 /*
  *  This file is part of pgn-extract: a Portable Game Notation (PGN) extractor.
- *  Copyright (C) 1994-2018 David J. Barnes
+ *  Copyright (C) 1994-2019 David J. Barnes
  *
  *  pgn-extract is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -161,6 +161,8 @@ which_output_format(const char *arg)
         { "elalg", ELALG},
         { "XLALG", XLALG},
         { "xlalg", XLALG},
+        { "XOLALG", XOLALG},
+        { "xolalg", XOLALG},
         { "uci", UCI},
         { "cm", CM},
         { "", SOURCE},
@@ -183,8 +185,11 @@ which_output_format(const char *arg)
              * a 6-piece suffix listing language-specific
              * letters to use in the output.
              */
-            if ((format == SAN || format == ELALG || format == XLALG) &&
-                    (strlen(arg) > format_prefix_len)) {
+            if ((format == SAN ||
+                 format == ELALG ||
+                 format == XLALG ||
+                 format == XOLALG) &&
+                (strlen(arg) > format_prefix_len)) {
                 set_output_piece_characters(&arg[format_prefix_len]);
             }
             return format;
@@ -210,6 +215,7 @@ output_file_suffix(OutputFormat format)
         case HALG:
         case ELALG:
         case XLALG:
+        case XOLALG:
         case UCI:
             return PGN_suffix;
         case EPD:
@@ -344,9 +350,11 @@ show_tags(FILE *outfp, char **Tags, int tags_length)
         }
     }
     /* Handle the remaining tags. */
-    for (tag_index = 0; tag_index < tags_length; tag_index++) {
-        if (copy_of_tags[tag_index] != NULL) {
-            output_tag(tag_index, copy_of_tags, outfp);
+    if(!GlobalState.only_output_wanted_tags) {
+        for (tag_index = 0; tag_index < tags_length; tag_index++) {
+            if (copy_of_tags[tag_index] != NULL) {
+                output_tag(tag_index, copy_of_tags, outfp);
+            }
         }
     }
     (void) free(copy_of_tags);
@@ -715,72 +723,85 @@ print_move(FILE *outputfile, unsigned move_number, Boolean print_move_number,
                     case LALG:
                     case ELALG:
                     case XLALG:
+                    case XOLALG:
                     case UCI:
                     {
                         char algebraic[MAX_MOVE_LEN + 1];
                         size_t ind = 0;
 
-                        /* Prefix with a piece name if ELALG. */
-                        if ((output_format == ELALG || output_format == XLALG) &&
-                                move_details->class == PIECE_MOVE) {
-                            strcpy(algebraic,
-                                    piece_str(move_details->piece_to_move));
-                            ind = strlen(algebraic);
-                        }
-                        /* Format the basics. */
-                        if (move_details->class != NULL_MOVE) {
-                            sprintf(&algebraic[ind],
-                                    "%c%c",
-                                    move_details->from_col,
-                                    move_details->from_rank);
-                            ind += 2;
-                            if (output_format == XLALG) {
-                                /* Add a separating - or x. */
-                                char separator;
-                                if (move_details->captured_piece != EMPTY) {
-                                    separator = 'x';
-                                }
-                                else {
-                                    separator = '-';
-                                }
-                                sprintf(&algebraic[ind],
-                                        "%c", separator);
-                                ind++;
-                            }
-                            sprintf(&algebraic[ind],
-                                    "%c%c",
-                                    move_details->to_col,
-                                    move_details->to_rank);
-                            ind += 2;
+                        if(output_format == XOLALG &&
+                           (move_details->class == KINGSIDE_CASTLE ||
+                            move_details->class == QUEENSIDE_CASTLE)) {
+                            strcpy(algebraic, (char *) move_text);
+                            ind += strlen((char *) algebraic);
                         }
                         else {
-                            strcpy(algebraic, NULL_MOVE_STRING);
-                            ind += strlen(NULL_MOVE_STRING);
-                        }
-                        switch (move_details->class) {
-                            case PAWN_MOVE:
-                            case KINGSIDE_CASTLE:
-                            case QUEENSIDE_CASTLE:
-                            case PIECE_MOVE:
-                            case NULL_MOVE:
-                                /* Nothing more to do at this stage. */
-                                break;
-                            case ENPASSANT_PAWN_MOVE:
-                                if (output_format == ELALG) {
-                                    strcat(algebraic, "ep");
-                                    ind += 2;
-                                }
-                                break;
-                            case PAWN_MOVE_WITH_PROMOTION:
-                                sprintf(&algebraic[ind],
-                                        "%s",
-                                        piece_str(move_details->promoted_piece));
+                            /* Prefix with a piece name if ELALG. */
+                            if ((output_format == ELALG ||
+                                 output_format == XLALG ||
+                                 output_format == XOLALG) &&
+                                    move_details->class == PIECE_MOVE) {
+                                strcpy(algebraic,
+                                        piece_str(move_details->piece_to_move));
                                 ind = strlen(algebraic);
-                                break;
-                            case UNKNOWN_MOVE:
-                                strcpy(algebraic, "???");
-                                ind += 3;
-                                break;
+                            }
+                            /* Format the basics. */
+                            if (move_details->class != NULL_MOVE) {
+                                sprintf(&algebraic[ind],
+                                        "%c%c",
+                                        move_details->from_col,
+                                        move_details->from_rank);
+
+                                ind += 2;
+                                if (output_format == XLALG ||
+                                    output_format == XOLALG) {
+                                    /* Add a separating - or x. */
+                                    char separator;
+                                    if (move_details->captured_piece != EMPTY) {
+                                        separator = 'x';
+                                    }
+                                    else {
+                                        separator = '-';
+                                    }
+                                    sprintf(&algebraic[ind],
+                                            "%c", separator);
+                                    ind++;
+                                }
+                                sprintf(&algebraic[ind],
+                                        "%c%c",
+                                        move_details->to_col,
+                                        move_details->to_rank);
+                                ind += 2;
+                            }
+                            else {
+                                strcpy(algebraic, NULL_MOVE_STRING);
+                                ind += strlen(NULL_MOVE_STRING);
+                            }
+                            switch (move_details->class) {
+                                case PAWN_MOVE:
+                                case KINGSIDE_CASTLE:
+                                case QUEENSIDE_CASTLE:
+                                case PIECE_MOVE:
+                                case NULL_MOVE:
+                                    /* Nothing more to do at this stage. */
+                                    break;
+                                case ENPASSANT_PAWN_MOVE:
+                                    if (output_format == ELALG) {
+                                        strcat(algebraic, "ep");
+                                        ind += 2;
+                                    }
+                                    break;
+                                case PAWN_MOVE_WITH_PROMOTION:
+                                    sprintf(&algebraic[ind],
+                                            "%s",
+                                            piece_str(move_details->promoted_piece));
+                                    ind = strlen(algebraic);
+                                    break;
+                                case UNKNOWN_MOVE:
+                                    strcpy(algebraic, "???");
+                                    ind += 3;
+                                    break;
+                            }
                         }
                         if (GlobalState.keep_checks) {
                             switch (move_details->check_status) {
@@ -1265,6 +1286,7 @@ format_game(Game *current_game, FILE *outputfile)
             case HALG:
             case ELALG:
             case XLALG:
+            case XOLALG:
             case UCI:
                 print_algebraic_game(current_game, outputfile, move_number, white_to_move,
                         final_board);
